﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.ServiceModel.DomainServices.Client;

namespace Insert_DeleteRecords_RIA
{
	public abstract partial class EntityCollectionViewBase<TEntity> : INotifyPropertyChanged, INotifyCollectionChanged where TEntity : Entity, new()
	{
		private readonly IEnumerable<TEntity> entities;
		private readonly EntityCollectionViewDelegates<TEntity> delegates;

		protected EntityCollectionViewBase(IEnumerable<TEntity> entities, EntityCollectionViewDelegates<TEntity> delegates)
		{
			this.entities = entities;
			this.delegates = delegates;

			var observableSourceCollection = this.entities as INotifyCollectionChanged;
			if (observableSourceCollection != null)
			{
				observableSourceCollection.CollectionChanged += OnEntitiesCollectionChanged;
			}

			var observableSource = this.entities as INotifyPropertyChanged;
			if (observableSource != null)
			{
				observableSource.PropertyChanged += OnEntitiesPropertyChanged;
			}
		}

		protected virtual void OnEntitiesPropertyChanged(object sender, PropertyChangedEventArgs e)
		{
			this.OnPropertyChanged(e);
		}

		protected virtual void OnPropertyChanged(PropertyChangedEventArgs args)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, args);
			}
		}

		/// <summary>Occurs when a property value changes.</summary>
		public event PropertyChangedEventHandler PropertyChanged;  

		protected virtual void OnEntitiesCollectionChanged(object sender, NotifyCollectionChangedEventArgs args)
		{
			this.OnCollectionChanged(args);
		}

		/// <summary>Occurs when the items list of the collection has changed, or the collection
		/// is reset.</summary>
		public event NotifyCollectionChangedEventHandler CollectionChanged;

		protected virtual void OnCollectionChanged(NotifyCollectionChangedEventArgs args)
		{
			if (this.CollectionChanged != null)
			{
				this.CollectionChanged(this, args);
			}
		}

		public event EventHandler<CancelableEntityEventArgs<TEntity>> AddingNew;
		public event EventHandler<EntityEventArgs<TEntity>> AddedNew;

		public event EventHandler<CancelEventArgs> CommittingNew;
		public event EventHandler<EntityEventArgs<TEntity>> CommittedNew;

		public event EventHandler<CancelEventArgs> CancellingNew;
		public event EventHandler<EntityEventArgs<TEntity>> CancelledNew;

		public event EventHandler<CancelableEntityEventArgs<TEntity>> EditingItem;
		public event EventHandler<EntityEventArgs<TEntity>> EditedItem;

		public event EventHandler<CancelEventArgs> CommittingEdit;
		public event EventHandler<EntityEventArgs<TEntity>> CommittedEdit;

		public event EventHandler<CancelEventArgs> CancellingEdit;
		public event EventHandler<EntityEventArgs<TEntity>> CancelledEdit;

		public event EventHandler<CancelableEntityEventArgs<TEntity>> Removing;
		public event EventHandler<EntityEventArgs<TEntity>> Removed;

		protected virtual void OnAddingNew(CancelableEntityEventArgs<TEntity> args)
		{
			if (this.AddingNew != null)
			{
				this.AddingNew(this, args);
			}
		}
  
		protected virtual void OnAddedNew(EntityEventArgs<TEntity> args)
		{
			if (this.AddedNew != null)
			{
				this.AddedNew(this, args);
			}
		}
  
		protected virtual void OnCommittingNew(CancelEventArgs args)
		{
			if (this.CommittingNew != null)
			{
				this.CommittingNew(this, args);
			}
		}
  
		protected virtual void OnCommittedNew(EntityEventArgs<TEntity> args)
		{
			if (this.CommittedNew != null)
			{
				this.CommittedNew(this, args);
			}
		}
  
		protected virtual void OnCancellingNew(CancelEventArgs args)
		{
			if (this.CancellingNew != null)
			{
				this.CancellingNew(this, args);
			}
		}
  
		protected virtual void OnCancelledNew(EntityEventArgs<TEntity> args)
		{
			if (this.CancelledNew != null)
			{
				this.CancelledNew(this, args);
			}
		}
  
		protected virtual void OnEditingItem(CancelableEntityEventArgs<TEntity> args)
		{
			if (this.EditingItem != null)
			{
				this.EditingItem(this, args);
			}
		}
  
		protected virtual void OnEditedItem(EntityEventArgs<TEntity> args)
		{
			if (this.EditedItem != null)
			{
				this.EditedItem(this, args);
			}
		}
  
		protected virtual void OnCommittingEdit(CancelEventArgs args)
		{
			if (this.CommittingEdit != null)
			{
				this.CommittingEdit(this, args);
			}
		}
  
		protected virtual void OnCommittedEdit(EntityEventArgs<TEntity> args)
		{
			if (this.CommittedEdit != null)
			{
				this.CommittedEdit(this, args);
			}
		}
  
		protected virtual void OnCancellingEdit(CancelEventArgs args)
		{
			if (this.CancellingEdit != null)
			{
				this.CancellingEdit(this, args);
			}
		}
  
		protected virtual void OnCancelledEdit(EntityEventArgs<TEntity> args)
		{
			if (this.CancelledEdit != null)
			{
				this.CancelledEdit(this, args);
			}
		}
  
		protected virtual void OnRemoving(CancelableEntityEventArgs<TEntity> args)
		{
			if (this.Removing != null)
			{
				this.Removing(this, args);
			}
		}
  
		protected virtual void OnRemoved(EntityEventArgs<TEntity> args)
		{
			if (this.Removed != null)
			{
				this.Removed(this, args);
			}
		}
	}
}
