﻿using Insert_DeleteRecords_RIA.Web;
using Microsoft.Practices.Composite.Presentation.Commands;
using System.Collections;
using System.ComponentModel;
using Telerik.Windows.Controls;

namespace Insert_DeleteRecords_RIA
{
    public class MyViewModel : ViewModelBase
    {
		private readonly CustomerDomainContext context;

		public ActionCommand SubmitChangesCommand { get; private set; }
		public ActionCommand RejectChangesCommand { get; private set; }

		private EntityCollectionViewBase<Customer> customers;
		private EntityCollectionViewBase<Order> orders;
		private EntityCollectionViewBase<Order_Detail> orderDetails;

        public MyViewModel()
        {
			this.context = new CustomerDomainContext();
            this.context.PropertyChanged += OnContextPropertyChanged;

			this.SubmitChangesCommand = new ActionCommand(() => this.context.SubmitChanges(), () => this.context.HasChanges);
			this.RejectChangesCommand = new ActionCommand(() => this.context.RejectChanges(), () => this.context.HasChanges);
		}

        void OnContextPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsLoading")
            {
                this.IsLoading = this.context.IsLoading;
            }
			else if (e.PropertyName == "IsSubmitting")
			{
				this.IsLoading = this.context.IsSubmitting;
			}
			else if (e.PropertyName == "HasChanges")
			{
				this.SubmitChangesCommand.RaiseCanExecuteChanged();
				this.RejectChangesCommand.RaiseCanExecuteChanged();
			}
        }

        public IEnumerable Customers 
        {
            get
            {
                if (this.customers == null)
                {
					this.customers = new EntitySetCollectionView<Customer>(this.context.Customers);

                    this.context.Load(this.context.GetCustomersQuery());
                }

				return customers;
            }
        }

		public IEnumerable OrderDetails
		{
			get
			{
				if (this.orderDetails == null)
				{
					this.orderDetails = new EntitySetCollectionView<Order_Detail>(this.context.Order_Details);

					this.context.Load(this.context.GetOrder_DetailsQuery());
				}

				return orderDetails;
			}
		}

		public IEnumerable Orders
		{
			get
			{
				if (this.orders == null)
				{
					this.orders = new EntitySetCollectionView<Order>(this.context.Orders);

					this.context.Load(this.context.GetOrdersQuery());
				}

				return orders;
			}
		}

        private bool isLoading;

        public bool IsLoading
        {
            get
            {
                return this.isLoading;
            }
            private set
            {
                if (this.isLoading != value)
                {
                    this.isLoading = value;
                    this.OnPropertyChanged("IsLoading");
                }
            }
        }
    }
}
